(function webpackUniversalModuleDefinition(root, factory) {
	if(typeof exports === 'object' && typeof module === 'object')
		module.exports = factory();
	else if(typeof define === 'function' && define.amd)
		define([], factory);
	else if(typeof exports === 'object')
		exports["MarkdownEditor"] = factory();
	else
		root["MarkdownEditor"] = factory();
})(self, () => {
return /******/ (() => { // webpackBootstrap
/******/ 	"use strict";
/******/ 	var __webpack_modules__ = ([
/* 0 */
/***/ (function(__unused_webpack_module, exports, __webpack_require__) {


var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", ({ value: true }));
exports.isCodeBlock = exports.isCodeBlockSpecialCase1 = void 0;
const turndown_1 = __importDefault(__webpack_require__(1));
class MarkdownEditor {
    constructor(box) {
        this.config = {
            enableAutoCompletion: false,
            bracketMatchPairs: [
                {
                    enabled: true,
                    left: '(',
                    right: ')',
                },
                {
                    enabled: true,
                    left: '[',
                    right: ']',
                },
                {
                    enabled: true,
                    left: '{',
                    right: '}',
                },
                {
                    enabled: true,
                    left: '``',
                    right: '`\n```',
                    backspace: '``',
                    moveBack: 4,
                    trigger: '`',
                    needLeftMatch: true,
                },
                {
                    enabled: true,
                    left: '`',
                    right: '`',
                },
                {
                    enabled: true,
                    left: '*',
                    right: '***',
                    moveBack: 2,
                    rollback: '*',
                    trigger: '*',
                    needLeftMatch: true,
                },
                {
                    enabled: true,
                    left: '_',
                    right: '___',
                    moveBack: 2,
                    rollback: '_',
                    trigger: '_',
                    needLeftMatch: true,
                },
                {
                    enabled: true,
                    left: '"',
                    right: '"',
                },
                {
                    enabled: false,
                    left: '\'',
                    right: '\'',
                },
                {
                    enabled: false,
                    left: '$$',
                    right: '\n\n$$',
                    backspace: '$$',
                    moveBack: 3,
                    trigger: '$',
                    needLeftMatch: true,
                },
                {
                    enabled: false,
                    left: '$',
                    right: '$',
                },
            ],
        };
        this.hotKeyMap = [
            {
                key: 'ctrl+`',
                action: (editor) => editor.code(),
            },
            {
                key: 'ctrl+b',
                action: (editor) => editor.bold(),
            },
            {
                key: 'ctrl+q',
                action: (editor) => editor.quote(),
            },
            {
                key: 'ctrl+k',
                action: (editor) => editor.link(),
            },
            {
                key: 'ctrl+shift+f',
                action: (editor) => editor.table(),
            },
            {
                key: 'shift+tab',
                action: (editor) => editor.decreaseIndent(),
            },
            {
                key: 'tab',
                action: (editor) => editor.increaseIndent(),
            },
        ];
        this.disabled = false;
        this.textbox = box;
    }
    focusEditor() {
        if (this.disabled)
            return;
        const editor = this.textbox;
        setTimeout(function () {
            editor.focus();
        }, 10);
    }
    getSelection() {
        const lo = this.textbox.selectionStart;
        const hi = this.textbox.selectionEnd;
        return this.textbox.value.substring(lo, hi);
    }
    hasSelection() {
        return this.textbox.selectionStart !== this.textbox.selectionEnd;
    }
    clearSelectionToStart() {
        if (!this.hasSelection()) {
            return;
        }
        this.textbox.selectionEnd = this.textbox.selectionStart;
    }
    clearSelectionToEnd() {
        if (!this.hasSelection()) {
            return;
        }
        this.textbox.selectionStart = this.textbox.selectionEnd;
    }
    replaceSelectionWith(replacement) {
        const diff = replacement.length - this.getSelection().length;
        const newStart = this.textbox.selectionStart;
        const newEnd = this.textbox.selectionEnd + diff;
        if (!document.execCommand('insertText', false, replacement)) {
            this.textbox.setRangeText(replacement);
        }
        this.textbox.selectionStart = newStart;
        this.textbox.selectionEnd = newEnd;
        this.focusEditor();
    }
    wrapSelectionWith(prefix, suffix, lineByline) {
        return this.wrapSelectionWithGenerator(() => prefix, () => suffix, lineByline);
    }
    wrapSelectionWithGenerator(prefixGenerator, suffixGenerator, lineByline) {
        const value = this.getSelection();
        if (value.length === 0) {
            this.insertToCaret(prefixGenerator(0) + suffixGenerator(0));
            return;
        }
        if (lineByline) {
            const lines = value.split('\n');
            let replacement = '';
            for (let i = 0; i < lines.length; i++) {
                if (lines[i].endsWith('\r')) {
                    lines[i] = lines[i].slice(0, -1);
                }
                if (lines[i].length > 0) {
                    replacement += prefixGenerator(i) + lines[i] + suffixGenerator(i);
                }
                if (i !== lines.length - 1) {
                    replacement += '\n';
                }
            }
            this.replaceSelectionWith(replacement);
        }
        else {
            this.replaceSelectionWith(prefixGenerator(0) + value + suffixGenerator(0));
        }
        this.focusEditor();
    }
    getCurrentLine() {
        const caretPosition = this.textbox.selectionStart;
        const lastLine = this.textbox.value.substring(0, caretPosition).lastIndexOf('\n');
        return this.textbox.value.substring(lastLine + 1, caretPosition);
    }
    getStringRelativeToCaret(offset) {
        if (offset < 0) {
            const start = this.textbox.selectionStart;
            return this.textbox.value.substring(start + offset, start);
        }
        const end = this.textbox.selectionEnd;
        return this.textbox.value.substring(end, end + offset);
    }
    insertToCaret(value) {
        if (!document.execCommand('insertText', false, value)) {
            const index = this.textbox.selectionEnd;
            this.textbox.setRangeText(value, index, index, 'end');
        }
        this.focusEditor();
    }
    moveCaretBy(count) {
        if (!this.hasSelection()) {
            if (count > 0) {
                this.textbox.selectionStart += count;
            }
            else {
                this.textbox.selectionEnd += count;
            }
        }
        else {
            this.textbox.selectionStart += count;
            this.textbox.selectionEnd += count;
        }
    }
    getDraftKey(app, userId, postId) {
        return app + '-draft-' + userId + '-' + postId;
    }
    saveDraft(app, userId, postId) {
        if (this.textbox.value.length > 4000) {
            return;
        }
        window.localStorage.setItem(this.getDraftKey(app, userId, postId), this.textbox.value);
    }
    loadDraft(app, userId, postId) {
        const value = window.localStorage.getItem(this.getDraftKey(app, userId, postId));
        if (value && !this.textbox.value) {
            this.textbox.value = value;
        }
    }
    removeDraft(app, userId, postId) {
        window.localStorage.removeItem(this.getDraftKey(app, userId, postId));
    }
    getConfigKey(app, userId) {
        return app + '-config-' + userId;
    }
    saveConfig(app, userId) {
        window.localStorage.setItem(this.getConfigKey(app, userId), JSON.stringify(this.config));
    }
    loadConfig(app, userId) {
        const value = window.localStorage.getItem(this.getConfigKey(app, userId));
        if (value) {
            this.config = Object.assign(Object.assign({}, this.config), JSON.parse(value));
        }
    }
    bracketMatch(input) {
        for (let i = 0; i < this.config.bracketMatchPairs.length; i++) {
            const current = this.config.bracketMatchPairs[i];
            if (current.enabled === false) {
                continue;
            }
            const leftMatched = this.getStringRelativeToCaret(-current.left.length) === current.left;
            const rightMatched = this.getStringRelativeToCaret(current.right.length) === current.right;
            if (input === current.right && rightMatched) {
                this.moveCaretBy(current.right.length);
                return true;
            }
            if (input === (current.trigger || current.left)) {
                if (this.hasSelection()) {
                    if (!current.needLeftMatch) {
                        this.wrapSelectionWith(current.left, current.right, false);
                        return true;
                    }
                    continue;
                }
                if (leftMatched && current.needLeftMatch) {
                    this.insertToCaret(current.right);
                    this.moveCaretBy(-(current.moveBack || current.right.length));
                    return true;
                }
                if (!current.needLeftMatch) {
                    this.insertToCaret(current.left + current.right);
                    this.moveCaretBy(-(current.moveBack || current.right.length));
                    return true;
                }
            }
        }
        return false;
    }
    bold() {
        this.wrapSelectionWith('**', '**', true);
        if (this.hasSelection()) {
            this.clearSelectionToEnd();
        }
        else {
            this.moveCaretBy(-2);
        }
    }
    link() {
        const selection = this.getSelection();
        if (selection) {
            try {
                new URL(selection);
                this.wrapSelectionWith('[](', ')', true);
                this.clearSelectionToStart();
                this.moveCaretBy(1);
                return;
            }
            catch (_a) {
            }
            this.wrapSelectionWith('[', ']()', true);
            this.clearSelectionToEnd();
            this.moveCaretBy(-1);
            return;
        }
        this.wrapSelectionWith('[', ']()', true);
        this.clearSelectionToEnd();
        this.moveCaretBy(-3);
    }
    quote() {
        this.wrapSelectionWith('> ', '', true);
        if (this.hasSelection()) {
            this.clearSelectionToEnd();
        }
    }
    code() {
        if (!this.hasSelection()) {
            this.insertToCaret('``');
            this.moveCaretBy(-1);
            return;
        }
        const selection = this.getSelection();
        const newlineIndex = selection.lastIndexOf('\n');
        if (newlineIndex > 0) {
            let suffix = '```\n';
            if (newlineIndex !== selection.length - 1) {
                suffix = '\n' + suffix;
            }
            this.wrapSelectionWith('```\n', suffix, false);
        }
        else {
            this.wrapSelectionWith('`', '`', false);
        }
        this.clearSelectionToEnd();
    }
    collapsibleCodeBlock() {
        let prefix = '<details>\n<summary>点击查看代码</summary>\n\n```\n';
        const suffix = '\n```\n</details>';
        if (!this.hasSelection()) {
            this.insertToCaret(prefix + suffix);
            this.moveCaretBy(-suffix.length);
            return;
        }
        const selectionStart = this.textbox.selectionStart;
        if (this.textbox.value[selectionStart - 1] !== '\n') {
            prefix = '\n' + prefix;
        }
        this.wrapSelectionWith(prefix, suffix, false);
        this.moveCaretBy(-suffix.length);
        this.clearSelectionToEnd();
    }
    taskLists() {
        let prefix = '- [ ] ';
        const suffix = '';
        if (!this.hasSelection()) {
            this.insertToCaret(prefix + suffix);
            this.moveCaretBy(-suffix.length);
            return;
        }
        const selectionStart = this.textbox.selectionStart;
        if (this.textbox.value[selectionStart - 1] !== '\n') {
            prefix = '\n' + prefix;
        }
        this.wrapSelectionWith(prefix, suffix, false);
        this.moveCaretBy(-suffix.length);
        this.clearSelectionToEnd();
    }
    table(col, row) {
        if (col == undefined && row == undefined) {
            if (this.hasSelection()) {
                const rows = this.getSelection()
                    .trimRight()
                    .split('\n')
                    .map(s => s.trim())
                    .map(s => (s.startsWith('|') ? s.slice(1) : s))
                    .map(s => (s.endsWith('|') ? s.slice(0, -1) : s))
                    .map(s => s.split('|'));
                this.buildTable(rows);
                return;
            }
            else {
                col = 2;
                row = 3;
            }
        }
        this.buildTable(new Array(col).fill('    ').map(c => new Array(row).fill(c)));
    }
    numberedList() {
        this.wrapSelectionWithGenerator(n => n + 1 + '. ', () => '', true);
    }
    bulletList() {
        this.wrapSelectionWith('* ', '', true);
    }
    buildTable(rows) {
        const hasTableHeaderSplitter = rows.length > 1 &&
            rows[1].every(i => i
                .trim()
                .split('')
                .every(c => c === '-' || c === ':'));
        if (hasTableHeaderSplitter) {
            rows.splice(1, 1);
        }
        rows = rows.map(r => r.map(col => col.trim()));
        rows = this.formatTable(rows);
        const col = hasTableHeaderSplitter ? rows[0].length : Math.max(...rows.map(r => r.length), 2);
        while (rows[0].length < col) {
            rows[0].push('header');
        }
        if (rows.length == 1) {
            rows.push(new Array(col).fill('    '));
        }
        rows = this.alignTable(rows);
        const headerLine = '|' + rows[0].join('|') + '|';
        const headerSplitter = '| ' + rows[0].map(s => new Array(s.length - 1).join('-')).join(' | ') + ' |';
        const content = rows
            .slice(1)
            .map(r => {
            r.length = col;
            return '|' + r.join('|') + '|';
        })
            .join('\n');
        this.replaceSelectionWith(headerLine + '\n' + headerSplitter + '\n' + content);
        this.clearSelectionToEnd();
        this.focusEditor();
    }
    formatTable(rows) {
        if (rows.length < 2) {
            return rows;
        }
        const placeholder = (num) => new Array(num === 0 ? 4 : num).fill(' ').join('');
        const diff = rows[0].length - Math.max(...rows.slice(1).map(r => r.length));
        if (diff > 0) {
            for (let i = 1; i < rows.length; i++) {
                const zippedRow = [];
                rows[0].forEach((cur, colIndex) => {
                    var _a;
                    return zippedRow.push(cur === ''
                        ? placeholder(rows[i - 1][colIndex].length)
                        : (_a = rows[i].shift()) !== null && _a !== void 0 ? _a : placeholder(rows[i - 1][colIndex].length));
                });
                rows[i] = zippedRow;
            }
            rows[0] = rows[0].map(cur => (cur === '' ? placeholder(0) : cur));
        }
        else {
            for (let i = 1; i < rows.length; i++) {
                let zippedRow = [];
                rows[0].forEach((cur, colIndex) => {
                    var _a;
                    let toAdd;
                    if (cur === '') {
                        if (rows[i].length > 0) {
                            rows[i].shift();
                            toAdd = '';
                        }
                        else {
                            toAdd = placeholder(rows[i - 1][colIndex].length);
                        }
                    }
                    else {
                        toAdd = (_a = rows[i].shift()) !== null && _a !== void 0 ? _a : placeholder(rows[i - 1][colIndex].length);
                        if (toAdd === '') {
                            toAdd = placeholder(rows[i - 1][colIndex].length);
                        }
                    }
                    zippedRow.push(toAdd);
                });
                rows[i] = zippedRow;
            }
            rows = rows.map(row => row.filter(s => s !== ''));
        }
        return rows;
    }
    alignTable(rows) {
        rows[0].forEach((cur, col) => {
            const max = Math.max(...rows.map(r => r[col].length), 4) + 2;
            rows.forEach(r => (r[col] = r[col].padEnd(max - 1).padStart(max)));
        });
        return rows;
    }
    increaseIndent() {
        if (this.hasSelection()) {
            this.wrapSelectionWith('  ', '', true);
        }
        else {
            this.insertToCaret('  ');
        }
    }
    decreaseIndent() {
        const lines = this.getSelection().split('\n');
        let trimString = '';
        for (let i = 0; i < lines.length; i++) {
            if (lines[i].startsWith('  ')) {
                lines[i] = lines[i].slice(2);
            }
            else if (lines[i].startsWith(' ')) {
                lines[i] = lines[i].slice(1);
            }
            if (lines[i].endsWith('\r')) {
                lines[i] = lines[i].slice(0, -1);
            }
            trimString += lines[i];
            if (i !== lines.length - 1) {
                trimString += '\n';
            }
        }
        this.replaceSelectionWith(trimString);
    }
    isPastingTable(data) {
        const typeSet = new Set(data.types);
        if (typeSet.has('text/html') && typeSet.has('text/plain')) {
            const doc = new DOMParser().parseFromString(data.getData('text/html'), 'text/html');
            if (doc.getElementsByTagName('table').length !== 1) {
                return false;
            }
            const text = data.getData('text/plain').trim();
            const rows = text.split(/[\n\u0085\u2028\u2029]|\r\n?/g);
            if (doc.querySelectorAll('tr').length !== rows.length) {
                return false;
            }
            const textMaxColumn = Math.max(...rows.map(r => r.split('\t').length));
            let htmlMaxColumn = 0;
            doc
                .querySelectorAll('table tr')
                .forEach((row) => (htmlMaxColumn = Math.max(htmlMaxColumn, row.cells.length)));
            return textMaxColumn === htmlMaxColumn;
        }
        return false;
    }
    isPastingCode(data) {
        const typeSet = new Set(data.types);
        if (typeSet.has('text/markdown')) {
            return false;
        }
        if (typeSet.has('text/html') && typeSet.has('text/plain')) {
            if (typeSet.has('vscode-editor-data')) {
                try {
                    const { mode } = JSON.parse(data.getData('vscode-editor-data'));
                    return mode !== 'markdown';
                }
                catch (e) {
                    return true;
                }
            }
            const doc = new DOMParser().parseFromString(data.getData('text/html'), 'text/html');
            return doc.getElementsByTagName('pre').length === 1;
        }
    }
    onKeydown(e) {
        if (e.isComposing || e.keyCode === 229) {
            return true;
        }
        for (let i = 0; i < this.hotKeyMap.length; i++) {
            let canApply = true;
            const keys = this.hotKeyMap[i].key.split('+');
            for (let j = 0; j < keys.length; j++) {
                if (keys[j].toLowerCase() === 'ctrl'.toLowerCase()) {
                    canApply = canApply && (e.ctrlKey || e.metaKey);
                }
                else if (keys[j].toLowerCase() === 'shift'.toLowerCase()) {
                    canApply = canApply && e.shiftKey;
                }
                else if (keys[j].toLowerCase() === 'alt'.toLowerCase()) {
                    canApply = canApply && e.altKey;
                }
                else {
                    canApply = canApply && e.key.toLowerCase() === keys[j].toLowerCase();
                }
            }
            if (canApply) {
                this.hotKeyMap[i].action(this);
                e.preventDefault();
                e.stopPropagation();
                return false;
            }
        }
        if (!this.config.enableAutoCompletion) {
            return true;
        }
        if (this.bracketMatch(e.key)) {
            e.preventDefault();
            e.stopPropagation();
            return false;
        }
        if (e.key === 'Enter') {
            const currentLine = this.getCurrentLine();
            const listMark = currentLine.substring(0, 2);
            if (listMark === '* ' || listMark === '- ') {
                this.insertToCaret('\n' + listMark);
                e.preventDefault();
                e.stopPropagation();
                return false;
            }
            const ol = currentLine.match(/^(\d+)([.)]) [^\n]*/);
            if (ol) {
                const nextIndex = parseInt(ol[1]) + 1;
                this.insertToCaret('\n' + nextIndex + ol[2] + ' ');
                e.preventDefault();
                e.stopPropagation();
                return false;
            }
        }
        return true;
    }
    onPaste(e) {
        const clipboardData = e.clipboardData;
        let preventDefault = false;
        try {
            if (clipboardData && clipboardData.items) {
                if (this.isPastingTable(clipboardData)) {
                    const rows = clipboardData
                        .getData('text/plain')
                        .trim()
                        .split(/[\n\u0085\u2028\u2029]|\r\n?/g)
                        .map(row => row.split('\t'));
                    this.buildTable(rows);
                    preventDefault = true;
                }
                else if (this.config.enableAutoCompletion && this.isPastingCode(clipboardData)) {
                    const code = clipboardData.getData('text/plain');
                    const editorData = clipboardData.getData('vscode-editor-data');
                    const { mode } = JSON.parse(editorData ? editorData : '{ "mode": "plaintext" }');
                    if (code.indexOf('\n') >= 0) {
                        this.insertToCaret('```' + (mode ? mode : 'plaintext') + '\n\n```');
                        this.moveCaretBy(-4);
                    }
                    else {
                        this.insertToCaret('``');
                        this.moveCaretBy(-1);
                    }
                    preventDefault = false;
                }
                else if (clipboardData.types.includes('text/html') && clipboardData.types.includes('docx/record')) {
                    const turnDownService = new turndown_1.default({
                        codeBlockStyle: 'fenced',
                        fence: '```',
                        headingStyle: 'atx',
                        linkStyle: 'inlined',
                        strongDelimiter: '**',
                        bulletListMarker: '-'
                    });
                    turnDownService.use(gfm);
                    const markdown = turnDownService.turndown(clipboardData.getData('text/html'));
                    this.insertToCaret(markdown);
                    preventDefault = true;
                }
            }
        }
        catch (_a) {
            preventDefault = false;
        }
        if (preventDefault) {
            e.preventDefault();
        }
    }
}
exports["default"] = MarkdownEditor;
const highlightRegExp = /highlight-(?:text|source)-([a-z0-9]+)/;
function highlightedCodeBlock(turndownService) {
    turndownService.addRule('highlightedCodeBlock', {
        filter: function (node) {
            var firstChild = node.firstChild;
            return (node.nodeName === 'DIV' &&
                highlightRegExp.test(node.className) &&
                firstChild &&
                firstChild.nodeName === 'PRE');
        },
        replacement: function (content, node, options) {
            var className = node.className || '';
            var language = (className.match(highlightRegExp) || [null, ''])[1];
            return ('\n\n' + options.fence + language + '\n' +
                node.firstChild.textContent +
                '\n' + options.fence + '\n\n');
        }
    });
}
function strikethrough(turndownService) {
    turndownService.addRule('strikethrough', {
        filter: ['del', 's'],
        replacement: function (content) {
            return '~~' + content + '~~';
        }
    });
}
var indexOf = Array.prototype.indexOf;
var every = Array.prototype.every;
var rules = {};
var alignMap = { left: ':---', right: '---:', center: ':---:' };
let isCodeBlock_ = isCodeBlock;
const tableShouldBeSkippedCache_ = new WeakMap();
function getAlignment(node) {
    return node ? (node.getAttribute('align') || node.style.textAlign || '').toLowerCase() : '';
}
function getBorder(alignment) {
    return alignment ? alignMap[alignment] : '---';
}
function getColumnAlignment(table, columnIndex) {
    var votes = {
        left: 0,
        right: 0,
        center: 0,
        '': 0,
    };
    var align = '';
    for (var i = 0; i < table.rows.length; ++i) {
        var row = table.rows[i];
        if (columnIndex < row.childNodes.length) {
            var cellAlignment = getAlignment(row.childNodes[columnIndex]);
            ++votes[cellAlignment];
            if (votes[cellAlignment] > votes[align]) {
                align = cellAlignment;
            }
        }
    }
    return align;
}
rules.tableCell = {
    filter: ['th', 'td'],
    replacement: function (content, node) {
        if (tableShouldBeSkipped(nodeParentTable(node)))
            return content;
        return cell(content, node);
    }
};
rules.tableRow = {
    filter: 'tr',
    replacement: function (content, node) {
        const parentTable = nodeParentTable(node);
        if (tableShouldBeSkipped(parentTable))
            return content;
        var borderCells = '';
        if (isHeadingRow(node)) {
            const colCount = tableColCount(parentTable);
            for (var i = 0; i < colCount; i++) {
                const childNode = i < node.childNodes.length ? node.childNodes[i] : null;
                var border = getBorder(getColumnAlignment(parentTable, i));
                borderCells += cell(border, childNode, i);
            }
        }
        return '\n' + content + (borderCells ? '\n' + borderCells : '');
    }
};
rules.table = {
    filter: function (node, options) {
        return node.nodeName === 'TABLE' && !tableShouldBeHtml(node, options);
    },
    replacement: function (content, node) {
        if (tableShouldBeSkipped(node))
            return content;
        content = content.replace(/\n+/g, '\n');
        var secondLine = content.trim().split('\n');
        if (secondLine.length >= 2)
            secondLine = secondLine[1];
        var secondLineIsDivider = /\| :?---/.test(secondLine);
        var columnCount = tableColCount(node);
        var emptyHeader = '';
        if (columnCount && !secondLineIsDivider) {
            emptyHeader = '|' + '     |'.repeat(columnCount) + '\n' + '|';
            for (var columnIndex = 0; columnIndex < columnCount; ++columnIndex) {
                emptyHeader += ' ' + getBorder(getColumnAlignment(node, columnIndex)) + ' |';
            }
        }
        const captionContent = node.caption ? node.caption.textContent || '' : '';
        const caption = captionContent ? `${captionContent}\n\n` : '';
        const tableContent = `${emptyHeader}${content}`.trimStart();
        return `\n\n${caption}${tableContent}\n\n`;
    }
};
rules.tableCaption = {
    filter: ['caption'],
    replacement: () => '',
};
rules.tableColgroup = {
    filter: ['colgroup', 'col'],
    replacement: () => '',
};
rules.tableSection = {
    filter: ['thead', 'tbody', 'tfoot'],
    replacement: function (content) {
        return content;
    }
};
function isHeadingRow(tr) {
    var parentNode = tr.parentNode;
    return (parentNode.nodeName === 'THEAD' ||
        (parentNode.firstChild === tr &&
            (parentNode.nodeName === 'TABLE' || isFirstTbody(parentNode)) &&
            every.call(tr.childNodes, function (n) { return n.nodeName === 'TH'; })));
}
function isFirstTbody(element) {
    var previousSibling = element.previousSibling;
    return (element.nodeName === 'TBODY' && (!previousSibling ||
        (previousSibling.nodeName === 'THEAD' &&
            /^\s*$/i.test(previousSibling.textContent))));
}
function cell(content, node = null, index = null) {
    if (index === null)
        index = indexOf.call(node.parentNode.childNodes, node);
    var prefix = ' ';
    if (index === 0)
        prefix = '| ';
    let filteredContent = content.trim().replace(/\n\r/g, '<br>').replace(/\n/g, "<br>");
    filteredContent = filteredContent.replace(/\|+/g, '\\|');
    while (filteredContent.length < 3)
        filteredContent += ' ';
    if (node)
        filteredContent = handleColSpan(filteredContent, node, ' ');
    return prefix + filteredContent + ' |';
}
function nodeContainsTable(node) {
    if (!node.childNodes)
        return false;
    for (let i = 0; i < node.childNodes.length; i++) {
        const child = node.childNodes[i];
        if (child.nodeName === 'TABLE')
            return true;
        if (nodeContainsTable(child))
            return true;
    }
    return false;
}
const nodeContains = (node, types) => {
    if (!node.childNodes)
        return false;
    for (let i = 0; i < node.childNodes.length; i++) {
        const child = node.childNodes[i];
        if (types === 'code' && isCodeBlock_(child))
            return true;
        if (types.includes(child.nodeName))
            return true;
        if (nodeContains(child, types))
            return true;
    }
    return false;
};
const tableShouldBeHtml = (tableNode, options) => {
    const possibleTags = [
        'UL',
        'OL',
        'H1',
        'H2',
        'H3',
        'H4',
        'H5',
        'H6',
        'HR',
        'BLOCKQUOTE',
    ];
    if (options.preserveNestedTables)
        possibleTags.push('TABLE');
    return nodeContains(tableNode, 'code') ||
        nodeContains(tableNode, possibleTags);
};
function tableShouldBeSkipped(tableNode) {
    const cached = tableShouldBeSkippedCache_.get(tableNode);
    if (cached !== undefined)
        return cached;
    const result = tableShouldBeSkipped_(tableNode);
    tableShouldBeSkippedCache_.set(tableNode, result);
    return result;
}
function tableShouldBeSkipped_(tableNode) {
    if (!tableNode)
        return true;
    if (!tableNode.rows)
        return true;
    if (tableNode.rows.length === 1 && tableNode.rows[0].childNodes.length <= 1)
        return true;
    return nodeContainsTable(tableNode);
}
function nodeParentTable(node) {
    let parent = node.parentNode;
    while (parent.nodeName !== 'TABLE') {
        parent = parent.parentNode;
        if (!parent)
            return null;
    }
    return parent;
}
function handleColSpan(content, node, emptyChar) {
    const colspan = node.getAttribute('colspan') || 1;
    for (let i = 1; i < colspan; i++) {
        content += ' | ' + emptyChar.repeat(3);
    }
    return content;
}
function tableColCount(node) {
    let maxColCount = 0;
    for (let i = 0; i < node.rows.length; i++) {
        const row = node.rows[i];
        const colCount = row.childNodes.length;
        if (colCount > maxColCount)
            maxColCount = colCount;
    }
    return maxColCount;
}
function tables(turndownService) {
    turndownService.keep(function (node) {
        return node.nodeName === 'TABLE' && tableShouldBeHtml(node, turndownService.options);
    });
    for (var key in rules)
        turndownService.addRule(key, rules[key]);
}
function taskListItems(turndownService) {
    turndownService.addRule('taskListItems', {
        filter: function (node) {
            return node.type === 'checkbox' && node.parentNode.nodeName === 'LI';
        },
        replacement: function (content, node) {
            return (node.checked ? '[x]' : '[ ]') + ' ';
        }
    });
}
function gfm(turndownService) {
    turndownService.use([
        highlightedCodeBlock,
        strikethrough,
        tables,
        taskListItems
    ]);
}
function isCodeBlockSpecialCase1(node) {
    const parent = node.parentNode;
    if (!parent)
        return false;
    return parent.classList && parent.classList.contains('code') && parent.nodeName === 'TD' && node.nodeName === 'PRE';
}
exports.isCodeBlockSpecialCase1 = isCodeBlockSpecialCase1;
function isCodeBlock(node) {
    if (isCodeBlockSpecialCase1(node))
        return true;
    return (node.nodeName === 'PRE' &&
        node.firstChild &&
        node.firstChild.nodeName === 'CODE');
}
exports.isCodeBlock = isCodeBlock;


/***/ }),
/* 1 */
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "default": () => (__WEBPACK_DEFAULT_EXPORT__)
/* harmony export */ });
function extend(destination) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];
    for (var key in source) {
      if (source.hasOwnProperty(key)) destination[key] = source[key];
    }
  }
  return destination;
}
function repeat(character, count) {
  return Array(count + 1).join(character);
}
function trimLeadingNewlines(string) {
  return string.replace(/^\n*/, '');
}
function trimTrailingNewlines(string) {
  // avoid match-at-end regexp bottleneck, see #370
  var indexEnd = string.length;
  while (indexEnd > 0 && string[indexEnd - 1] === '\n') indexEnd--;
  return string.substring(0, indexEnd);
}
var blockElements = ['ADDRESS', 'ARTICLE', 'ASIDE', 'AUDIO', 'BLOCKQUOTE', 'BODY', 'CANVAS', 'CENTER', 'DD', 'DIR', 'DIV', 'DL', 'DT', 'FIELDSET', 'FIGCAPTION', 'FIGURE', 'FOOTER', 'FORM', 'FRAMESET', 'H1', 'H2', 'H3', 'H4', 'H5', 'H6', 'HEADER', 'HGROUP', 'HR', 'HTML', 'ISINDEX', 'LI', 'MAIN', 'MENU', 'NAV', 'NOFRAMES', 'NOSCRIPT', 'OL', 'OUTPUT', 'P', 'PRE', 'SECTION', 'TABLE', 'TBODY', 'TD', 'TFOOT', 'TH', 'THEAD', 'TR', 'UL'];
function isBlock(node) {
  return is(node, blockElements);
}
var voidElements = ['AREA', 'BASE', 'BR', 'COL', 'COMMAND', 'EMBED', 'HR', 'IMG', 'INPUT', 'KEYGEN', 'LINK', 'META', 'PARAM', 'SOURCE', 'TRACK', 'WBR'];
function isVoid(node) {
  return is(node, voidElements);
}
function hasVoid(node) {
  return has(node, voidElements);
}
var meaningfulWhenBlankElements = ['A', 'TABLE', 'THEAD', 'TBODY', 'TFOOT', 'TH', 'TD', 'IFRAME', 'SCRIPT', 'AUDIO', 'VIDEO'];
function isMeaningfulWhenBlank(node) {
  return is(node, meaningfulWhenBlankElements);
}
function hasMeaningfulWhenBlank(node) {
  return has(node, meaningfulWhenBlankElements);
}
function is(node, tagNames) {
  return tagNames.indexOf(node.nodeName) >= 0;
}
function has(node, tagNames) {
  return node.getElementsByTagName && tagNames.some(function (tagName) {
    return node.getElementsByTagName(tagName).length;
  });
}
var rules = {};
rules.paragraph = {
  filter: 'p',
  replacement: function (content) {
    return '\n\n' + content + '\n\n';
  }
};
rules.lineBreak = {
  filter: 'br',
  replacement: function (content, node, options) {
    return options.br + '\n';
  }
};
rules.heading = {
  filter: ['h1', 'h2', 'h3', 'h4', 'h5', 'h6'],
  replacement: function (content, node, options) {
    var hLevel = Number(node.nodeName.charAt(1));
    if (options.headingStyle === 'setext' && hLevel < 3) {
      var underline = repeat(hLevel === 1 ? '=' : '-', content.length);
      return '\n\n' + content + '\n' + underline + '\n\n';
    } else {
      return '\n\n' + repeat('#', hLevel) + ' ' + content + '\n\n';
    }
  }
};
rules.blockquote = {
  filter: 'blockquote',
  replacement: function (content) {
    content = content.replace(/^\n+|\n+$/g, '');
    content = content.replace(/^/gm, '> ');
    return '\n\n' + content + '\n\n';
  }
};
rules.list = {
  filter: ['ul', 'ol'],
  replacement: function (content, node) {
    var parent = node.parentNode;
    if (parent.nodeName === 'LI' && parent.lastElementChild === node) {
      return '\n' + content;
    } else {
      return '\n\n' + content + '\n\n';
    }
  }
};
rules.listItem = {
  filter: 'li',
  replacement: function (content, node, options) {
    content = content.replace(/^\n+/, '') // remove leading newlines
    .replace(/\n+$/, '\n') // replace trailing newlines with just a single one
    .replace(/\n/gm, '\n    '); // indent
    var prefix = options.bulletListMarker + '   ';
    var parent = node.parentNode;
    if (parent.nodeName === 'OL') {
      var start = parent.getAttribute('start');
      var index = Array.prototype.indexOf.call(parent.children, node);
      prefix = (start ? Number(start) + index : index + 1) + '.  ';
    }
    return prefix + content + (node.nextSibling && !/\n$/.test(content) ? '\n' : '');
  }
};
rules.indentedCodeBlock = {
  filter: function (node, options) {
    return options.codeBlockStyle === 'indented' && node.nodeName === 'PRE' && node.firstChild && node.firstChild.nodeName === 'CODE';
  },
  replacement: function (content, node, options) {
    return '\n\n    ' + node.firstChild.textContent.replace(/\n/g, '\n    ') + '\n\n';
  }
};
rules.fencedCodeBlock = {
  filter: function (node, options) {
    return options.codeBlockStyle === 'fenced' && node.nodeName === 'PRE' && node.firstChild && node.firstChild.nodeName === 'CODE';
  },
  replacement: function (content, node, options) {
    var className = node.firstChild.getAttribute('class') || '';
    var language = (className.match(/language-(\S+)/) || [null, ''])[1];
    var code = node.firstChild.textContent;
    var fenceChar = options.fence.charAt(0);
    var fenceSize = 3;
    var fenceInCodeRegex = new RegExp('^' + fenceChar + '{3,}', 'gm');
    var match;
    while (match = fenceInCodeRegex.exec(code)) {
      if (match[0].length >= fenceSize) {
        fenceSize = match[0].length + 1;
      }
    }
    var fence = repeat(fenceChar, fenceSize);
    return '\n\n' + fence + language + '\n' + code.replace(/\n$/, '') + '\n' + fence + '\n\n';
  }
};
rules.horizontalRule = {
  filter: 'hr',
  replacement: function (content, node, options) {
    return '\n\n' + options.hr + '\n\n';
  }
};
rules.inlineLink = {
  filter: function (node, options) {
    return options.linkStyle === 'inlined' && node.nodeName === 'A' && node.getAttribute('href');
  },
  replacement: function (content, node) {
    var href = node.getAttribute('href');
    if (href) href = href.replace(/([()])/g, '\\$1');
    var title = cleanAttribute(node.getAttribute('title'));
    if (title) title = ' "' + title.replace(/"/g, '\\"') + '"';
    return '[' + content + '](' + href + title + ')';
  }
};
rules.referenceLink = {
  filter: function (node, options) {
    return options.linkStyle === 'referenced' && node.nodeName === 'A' && node.getAttribute('href');
  },
  replacement: function (content, node, options) {
    var href = node.getAttribute('href');
    var title = cleanAttribute(node.getAttribute('title'));
    if (title) title = ' "' + title + '"';
    var replacement;
    var reference;
    switch (options.linkReferenceStyle) {
      case 'collapsed':
        replacement = '[' + content + '][]';
        reference = '[' + content + ']: ' + href + title;
        break;
      case 'shortcut':
        replacement = '[' + content + ']';
        reference = '[' + content + ']: ' + href + title;
        break;
      default:
        var id = this.references.length + 1;
        replacement = '[' + content + '][' + id + ']';
        reference = '[' + id + ']: ' + href + title;
    }
    this.references.push(reference);
    return replacement;
  },
  references: [],
  append: function (options) {
    var references = '';
    if (this.references.length) {
      references = '\n\n' + this.references.join('\n') + '\n\n';
      this.references = []; // Reset references
    }

    return references;
  }
};
rules.emphasis = {
  filter: ['em', 'i'],
  replacement: function (content, node, options) {
    if (!content.trim()) return '';
    return options.emDelimiter + content + options.emDelimiter;
  }
};
rules.strong = {
  filter: ['strong', 'b'],
  replacement: function (content, node, options) {
    if (!content.trim()) return '';
    return options.strongDelimiter + content + options.strongDelimiter;
  }
};
rules.code = {
  filter: function (node) {
    var hasSiblings = node.previousSibling || node.nextSibling;
    var isCodeBlock = node.parentNode.nodeName === 'PRE' && !hasSiblings;
    return node.nodeName === 'CODE' && !isCodeBlock;
  },
  replacement: function (content) {
    if (!content) return '';
    content = content.replace(/\r?\n|\r/g, ' ');
    var extraSpace = /^`|^ .*?[^ ].* $|`$/.test(content) ? ' ' : '';
    var delimiter = '`';
    var matches = content.match(/`+/gm) || [];
    while (matches.indexOf(delimiter) !== -1) delimiter = delimiter + '`';
    return delimiter + extraSpace + content + extraSpace + delimiter;
  }
};
rules.image = {
  filter: 'img',
  replacement: function (content, node) {
    var alt = cleanAttribute(node.getAttribute('alt'));
    var src = node.getAttribute('src') || '';
    var title = cleanAttribute(node.getAttribute('title'));
    var titlePart = title ? ' "' + title + '"' : '';
    return src ? '![' + alt + ']' + '(' + src + titlePart + ')' : '';
  }
};
function cleanAttribute(attribute) {
  return attribute ? attribute.replace(/(\n+\s*)+/g, '\n') : '';
}

/**
 * Manages a collection of rules used to convert HTML to Markdown
 */

function Rules(options) {
  this.options = options;
  this._keep = [];
  this._remove = [];
  this.blankRule = {
    replacement: options.blankReplacement
  };
  this.keepReplacement = options.keepReplacement;
  this.defaultRule = {
    replacement: options.defaultReplacement
  };
  this.array = [];
  for (var key in options.rules) this.array.push(options.rules[key]);
}
Rules.prototype = {
  add: function (key, rule) {
    this.array.unshift(rule);
  },
  keep: function (filter) {
    this._keep.unshift({
      filter: filter,
      replacement: this.keepReplacement
    });
  },
  remove: function (filter) {
    this._remove.unshift({
      filter: filter,
      replacement: function () {
        return '';
      }
    });
  },
  forNode: function (node) {
    if (node.isBlank) return this.blankRule;
    var rule;
    if (rule = findRule(this.array, node, this.options)) return rule;
    if (rule = findRule(this._keep, node, this.options)) return rule;
    if (rule = findRule(this._remove, node, this.options)) return rule;
    return this.defaultRule;
  },
  forEach: function (fn) {
    for (var i = 0; i < this.array.length; i++) fn(this.array[i], i);
  }
};
function findRule(rules, node, options) {
  for (var i = 0; i < rules.length; i++) {
    var rule = rules[i];
    if (filterValue(rule, node, options)) return rule;
  }
  return void 0;
}
function filterValue(rule, node, options) {
  var filter = rule.filter;
  if (typeof filter === 'string') {
    if (filter === node.nodeName.toLowerCase()) return true;
  } else if (Array.isArray(filter)) {
    if (filter.indexOf(node.nodeName.toLowerCase()) > -1) return true;
  } else if (typeof filter === 'function') {
    if (filter.call(rule, node, options)) return true;
  } else {
    throw new TypeError('`filter` needs to be a string, array, or function');
  }
}

/**
 * The collapseWhitespace function is adapted from collapse-whitespace
 * by Luc Thevenard.
 *
 * The MIT License (MIT)
 *
 * Copyright (c) 2014 Luc Thevenard <lucthevenard@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

/**
 * collapseWhitespace(options) removes extraneous whitespace from an the given element.
 *
 * @param {Object} options
 */
function collapseWhitespace(options) {
  var element = options.element;
  var isBlock = options.isBlock;
  var isVoid = options.isVoid;
  var isPre = options.isPre || function (node) {
    return node.nodeName === 'PRE';
  };
  if (!element.firstChild || isPre(element)) return;
  var prevText = null;
  var keepLeadingWs = false;
  var prev = null;
  var node = next(prev, element, isPre);
  while (node !== element) {
    if (node.nodeType === 3 || node.nodeType === 4) {
      // Node.TEXT_NODE or Node.CDATA_SECTION_NODE
      var text = node.data.replace(/[ \r\n\t]+/g, ' ');
      if ((!prevText || / $/.test(prevText.data)) && !keepLeadingWs && text[0] === ' ') {
        text = text.substr(1);
      }

      // `text` might be empty at this point.
      if (!text) {
        node = remove(node);
        continue;
      }
      node.data = text;
      prevText = node;
    } else if (node.nodeType === 1) {
      // Node.ELEMENT_NODE
      if (isBlock(node) || node.nodeName === 'BR') {
        if (prevText) {
          prevText.data = prevText.data.replace(/ $/, '');
        }
        prevText = null;
        keepLeadingWs = false;
      } else if (isVoid(node) || isPre(node)) {
        // Avoid trimming space around non-block, non-BR void elements and inline PRE.
        prevText = null;
        keepLeadingWs = true;
      } else if (prevText) {
        // Drop protection if set previously.
        keepLeadingWs = false;
      }
    } else {
      node = remove(node);
      continue;
    }
    var nextNode = next(prev, node, isPre);
    prev = node;
    node = nextNode;
  }
  if (prevText) {
    prevText.data = prevText.data.replace(/ $/, '');
    if (!prevText.data) {
      remove(prevText);
    }
  }
}

/**
 * remove(node) removes the given node from the DOM and returns the
 * next node in the sequence.
 *
 * @param {Node} node
 * @return {Node} node
 */
function remove(node) {
  var next = node.nextSibling || node.parentNode;
  node.parentNode.removeChild(node);
  return next;
}

/**
 * next(prev, current, isPre) returns the next node in the sequence, given the
 * current and previous nodes.
 *
 * @param {Node} prev
 * @param {Node} current
 * @param {Function} isPre
 * @return {Node}
 */
function next(prev, current, isPre) {
  if (prev && prev.parentNode === current || isPre(current)) {
    return current.nextSibling || current.parentNode;
  }
  return current.firstChild || current.nextSibling || current.parentNode;
}

/*
 * Set up window for Node.js
 */

var root = typeof window !== 'undefined' ? window : {};

/*
 * Parsing HTML strings
 */

function canParseHTMLNatively() {
  var Parser = root.DOMParser;
  var canParse = false;

  // Adapted from https://gist.github.com/1129031
  // Firefox/Opera/IE throw errors on unsupported types
  try {
    // WebKit returns null on unsupported types
    if (new Parser().parseFromString('', 'text/html')) {
      canParse = true;
    }
  } catch (e) {}
  return canParse;
}
function createHTMLParser() {
  var Parser = function () {};
  {
    if (shouldUseActiveX()) {
      Parser.prototype.parseFromString = function (string) {
        var doc = new window.ActiveXObject('htmlfile');
        doc.designMode = 'on'; // disable on-page scripts
        doc.open();
        doc.write(string);
        doc.close();
        return doc;
      };
    } else {
      Parser.prototype.parseFromString = function (string) {
        var doc = document.implementation.createHTMLDocument('');
        doc.open();
        doc.write(string);
        doc.close();
        return doc;
      };
    }
  }
  return Parser;
}
function shouldUseActiveX() {
  var useActiveX = false;
  try {
    document.implementation.createHTMLDocument('').open();
  } catch (e) {
    if (root.ActiveXObject) useActiveX = true;
  }
  return useActiveX;
}
var HTMLParser = canParseHTMLNatively() ? root.DOMParser : createHTMLParser();
function RootNode(input, options) {
  var root;
  if (typeof input === 'string') {
    var doc = htmlParser().parseFromString(
    // DOM parsers arrange elements in the <head> and <body>.
    // Wrapping in a custom element ensures elements are reliably arranged in
    // a single element.
    '<x-turndown id="turndown-root">' + input + '</x-turndown>', 'text/html');
    root = doc.getElementById('turndown-root');
  } else {
    root = input.cloneNode(true);
  }
  collapseWhitespace({
    element: root,
    isBlock: isBlock,
    isVoid: isVoid,
    isPre: options.preformattedCode ? isPreOrCode : null
  });
  return root;
}
var _htmlParser;
function htmlParser() {
  _htmlParser = _htmlParser || new HTMLParser();
  return _htmlParser;
}
function isPreOrCode(node) {
  return node.nodeName === 'PRE' || node.nodeName === 'CODE';
}
function Node(node, options) {
  node.isBlock = isBlock(node);
  node.isCode = node.nodeName === 'CODE' || node.parentNode.isCode;
  node.isBlank = isBlank(node);
  node.flankingWhitespace = flankingWhitespace(node, options);
  return node;
}
function isBlank(node) {
  return !isVoid(node) && !isMeaningfulWhenBlank(node) && /^\s*$/i.test(node.textContent) && !hasVoid(node) && !hasMeaningfulWhenBlank(node);
}
function flankingWhitespace(node, options) {
  if (node.isBlock || options.preformattedCode && node.isCode) {
    return {
      leading: '',
      trailing: ''
    };
  }
  var edges = edgeWhitespace(node.textContent);

  // abandon leading ASCII WS if left-flanked by ASCII WS
  if (edges.leadingAscii && isFlankedByWhitespace('left', node, options)) {
    edges.leading = edges.leadingNonAscii;
  }

  // abandon trailing ASCII WS if right-flanked by ASCII WS
  if (edges.trailingAscii && isFlankedByWhitespace('right', node, options)) {
    edges.trailing = edges.trailingNonAscii;
  }
  return {
    leading: edges.leading,
    trailing: edges.trailing
  };
}
function edgeWhitespace(string) {
  var m = string.match(/^(([ \t\r\n]*)(\s*))(?:(?=\S)[\s\S]*\S)?((\s*?)([ \t\r\n]*))$/);
  return {
    leading: m[1],
    // whole string for whitespace-only strings
    leadingAscii: m[2],
    leadingNonAscii: m[3],
    trailing: m[4],
    // empty for whitespace-only strings
    trailingNonAscii: m[5],
    trailingAscii: m[6]
  };
}
function isFlankedByWhitespace(side, node, options) {
  var sibling;
  var regExp;
  var isFlanked;
  if (side === 'left') {
    sibling = node.previousSibling;
    regExp = / $/;
  } else {
    sibling = node.nextSibling;
    regExp = /^ /;
  }
  if (sibling) {
    if (sibling.nodeType === 3) {
      isFlanked = regExp.test(sibling.nodeValue);
    } else if (options.preformattedCode && sibling.nodeName === 'CODE') {
      isFlanked = false;
    } else if (sibling.nodeType === 1 && !isBlock(sibling)) {
      isFlanked = regExp.test(sibling.textContent);
    }
  }
  return isFlanked;
}
var reduce = Array.prototype.reduce;
var escapes = [[/\\/g, '\\\\'], [/\*/g, '\\*'], [/^-/g, '\\-'], [/^\+ /g, '\\+ '], [/^(=+)/g, '\\$1'], [/^(#{1,6}) /g, '\\$1 '], [/`/g, '\\`'], [/^~~~/g, '\\~~~'], [/\[/g, '\\['], [/\]/g, '\\]'], [/^>/g, '\\>'], [/_/g, '\\_'], [/^(\d+)\. /g, '$1\\. ']];
function TurndownService(options) {
  if (!(this instanceof TurndownService)) return new TurndownService(options);
  var defaults = {
    rules: rules,
    headingStyle: 'setext',
    hr: '* * *',
    bulletListMarker: '*',
    codeBlockStyle: 'indented',
    fence: '```',
    emDelimiter: '_',
    strongDelimiter: '**',
    linkStyle: 'inlined',
    linkReferenceStyle: 'full',
    br: '  ',
    preformattedCode: false,
    blankReplacement: function (content, node) {
      return node.isBlock ? '\n\n' : '';
    },
    keepReplacement: function (content, node) {
      return node.isBlock ? '\n\n' + node.outerHTML + '\n\n' : node.outerHTML;
    },
    defaultReplacement: function (content, node) {
      return node.isBlock ? '\n\n' + content + '\n\n' : content;
    }
  };
  this.options = extend({}, defaults, options);
  this.rules = new Rules(this.options);
}
TurndownService.prototype = {
  /**
   * The entry point for converting a string or DOM node to Markdown
   * @public
   * @param {String|HTMLElement} input The string or DOM node to convert
   * @returns A Markdown representation of the input
   * @type String
   */

  turndown: function (input) {
    if (!canConvert(input)) {
      throw new TypeError(input + ' is not a string, or an element/document/fragment node.');
    }
    if (input === '') return '';
    var output = process.call(this, new RootNode(input, this.options));
    return postProcess.call(this, output);
  },
  /**
   * Add one or more plugins
   * @public
   * @param {Function|Array} plugin The plugin or array of plugins to add
   * @returns The Turndown instance for chaining
   * @type Object
   */

  use: function (plugin) {
    if (Array.isArray(plugin)) {
      for (var i = 0; i < plugin.length; i++) this.use(plugin[i]);
    } else if (typeof plugin === 'function') {
      plugin(this);
    } else {
      throw new TypeError('plugin must be a Function or an Array of Functions');
    }
    return this;
  },
  /**
   * Adds a rule
   * @public
   * @param {String} key The unique key of the rule
   * @param {Object} rule The rule
   * @returns The Turndown instance for chaining
   * @type Object
   */

  addRule: function (key, rule) {
    this.rules.add(key, rule);
    return this;
  },
  /**
   * Keep a node (as HTML) that matches the filter
   * @public
   * @param {String|Array|Function} filter The unique key of the rule
   * @returns The Turndown instance for chaining
   * @type Object
   */

  keep: function (filter) {
    this.rules.keep(filter);
    return this;
  },
  /**
   * Remove a node that matches the filter
   * @public
   * @param {String|Array|Function} filter The unique key of the rule
   * @returns The Turndown instance for chaining
   * @type Object
   */

  remove: function (filter) {
    this.rules.remove(filter);
    return this;
  },
  /**
   * Escapes Markdown syntax
   * @public
   * @param {String} string The string to escape
   * @returns A string with Markdown syntax escaped
   * @type String
   */

  escape: function (string) {
    return escapes.reduce(function (accumulator, escape) {
      return accumulator.replace(escape[0], escape[1]);
    }, string);
  }
};

/**
 * Reduces a DOM node down to its Markdown string equivalent
 * @private
 * @param {HTMLElement} parentNode The node to convert
 * @returns A Markdown representation of the node
 * @type String
 */

function process(parentNode) {
  var self = this;
  return reduce.call(parentNode.childNodes, function (output, node) {
    node = new Node(node, self.options);
    var replacement = '';
    if (node.nodeType === 3) {
      replacement = node.isCode ? node.nodeValue : self.escape(node.nodeValue);
    } else if (node.nodeType === 1) {
      replacement = replacementForNode.call(self, node);
    }
    return join(output, replacement);
  }, '');
}

/**
 * Appends strings as each rule requires and trims the output
 * @private
 * @param {String} output The conversion output
 * @returns A trimmed version of the ouput
 * @type String
 */

function postProcess(output) {
  var self = this;
  this.rules.forEach(function (rule) {
    if (typeof rule.append === 'function') {
      output = join(output, rule.append(self.options));
    }
  });
  return output.replace(/^[\t\r\n]+/, '').replace(/[\t\r\n\s]+$/, '');
}

/**
 * Converts an element node to its Markdown equivalent
 * @private
 * @param {HTMLElement} node The node to convert
 * @returns A Markdown representation of the node
 * @type String
 */

function replacementForNode(node) {
  var rule = this.rules.forNode(node);
  var content = process.call(this, node);
  var whitespace = node.flankingWhitespace;
  if (whitespace.leading || whitespace.trailing) content = content.trim();
  return whitespace.leading + rule.replacement(content, node, this.options) + whitespace.trailing;
}

/**
 * Joins replacement to the current output with appropriate number of new lines
 * @private
 * @param {String} output The current conversion output
 * @param {String} replacement The string to append to the output
 * @returns Joined output
 * @type String
 */

function join(output, replacement) {
  var s1 = trimTrailingNewlines(output);
  var s2 = trimLeadingNewlines(replacement);
  var nls = Math.max(output.length - s1.length, replacement.length - s2.length);
  var separator = '\n\n'.substring(0, nls);
  return s1 + separator + s2;
}

/**
 * Determines whether an input can be converted
 * @private
 * @param {String|HTMLElement} input Describe this parameter
 * @returns Describe what it returns
 * @type String|Object|Array|Boolean|Number
 */

function canConvert(input) {
  return input != null && (typeof input === 'string' || input.nodeType && (input.nodeType === 1 || input.nodeType === 9 || input.nodeType === 11));
}
/* harmony default export */ const __WEBPACK_DEFAULT_EXPORT__ = (TurndownService);

/***/ })
/******/ 	]);
/************************************************************************/
/******/ 	// The module cache
/******/ 	var __webpack_module_cache__ = {};
/******/ 	
/******/ 	// The require function
/******/ 	function __webpack_require__(moduleId) {
/******/ 		// Check if module is in cache
/******/ 		var cachedModule = __webpack_module_cache__[moduleId];
/******/ 		if (cachedModule !== undefined) {
/******/ 			return cachedModule.exports;
/******/ 		}
/******/ 		// Create a new module (and put it into the cache)
/******/ 		var module = __webpack_module_cache__[moduleId] = {
/******/ 			// no module.id needed
/******/ 			// no module.loaded needed
/******/ 			exports: {}
/******/ 		};
/******/ 	
/******/ 		// Execute the module function
/******/ 		__webpack_modules__[moduleId].call(module.exports, module, module.exports, __webpack_require__);
/******/ 	
/******/ 		// Return the exports of the module
/******/ 		return module.exports;
/******/ 	}
/******/ 	
/************************************************************************/
/******/ 	/* webpack/runtime/define property getters */
/******/ 	(() => {
/******/ 		// define getter functions for harmony exports
/******/ 		__webpack_require__.d = (exports, definition) => {
/******/ 			for(var key in definition) {
/******/ 				if(__webpack_require__.o(definition, key) && !__webpack_require__.o(exports, key)) {
/******/ 					Object.defineProperty(exports, key, { enumerable: true, get: definition[key] });
/******/ 				}
/******/ 			}
/******/ 		};
/******/ 	})();
/******/ 	
/******/ 	/* webpack/runtime/hasOwnProperty shorthand */
/******/ 	(() => {
/******/ 		__webpack_require__.o = (obj, prop) => (Object.prototype.hasOwnProperty.call(obj, prop))
/******/ 	})();
/******/ 	
/******/ 	/* webpack/runtime/make namespace object */
/******/ 	(() => {
/******/ 		// define __esModule on exports
/******/ 		__webpack_require__.r = (exports) => {
/******/ 			if(typeof Symbol !== 'undefined' && Symbol.toStringTag) {
/******/ 				Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });
/******/ 			}
/******/ 			Object.defineProperty(exports, '__esModule', { value: true });
/******/ 		};
/******/ 	})();
/******/ 	
/************************************************************************/
/******/ 	
/******/ 	// startup
/******/ 	// Load entry module and return exports
/******/ 	// This entry module is referenced by other modules so it can't be inlined
/******/ 	var __webpack_exports__ = __webpack_require__(0);
/******/ 	__webpack_exports__ = __webpack_exports__["default"];
/******/ 	
/******/ 	return __webpack_exports__;
/******/ })()
;
});